/*  this sketch has an ocean/waves theme.  It lets the user start the vizulization by dragging and dropping an mp3 file onto the stage.
The sound waves eminate from the mouse Y position.  */ 
//import the ddf.minim libraries to deal with sound
import ddf.minim.analysis.*;
import ddf.minim.*;
//import these two java libraries to deal with drag and drop files
import java.awt.dnd.*;
import java.awt.datatransfer.*;

//declare a number of variables
int drops;
int num = 600;
int j = 0;
float r = -.1;
boolean loaded = false;
//create an array to hold the various waves
Waves[] wave = new Waves[num];
//create font and image variables
PFont font;
PImage gradient;
//these two commands are needed to get the sound player and the analyisis
AudioPlayer song;
FFT fft;


void setup() {
  size (1024, 800);
  font = loadFont("Arial-Black-48.vlw");
  gradient = loadImage("gradient.png");
  smooth();
  //here we fill the wave array with instances of Waves
  for (int i = 0; i < num; i++){
    wave[i] = new Waves();
  }
  //start the Minim plugin
  Minim.start(this);
  frameRate(20);
  delay(500);
}


void draw() {
  //draw the ocean
  landscape();
  //wait until someone drops a file onto the screen to begin all below
  if (loaded) {
    //start analyzing a mono mix of the two channels from the song
    fft.forward(song.mix);
    //display the waves
    for(int h = 0; h < num; h ++) {
      wave[h].display();
    }
    //go thru the fft "bands" and update the waves with x, y, and power values
    for(int i = 0; i < fft.avgSize(); i++) {
       if (fft.getAvg(i) > 10) {
         wave[j].update(mouseX - 200 + i*20, height, fft.getAvg(i)*i/2);
         if (j< num - 1) j++;
         else j = 0;
       }
     }
   }
   //draw the sky and the sun
   noStroke();
   fill(50, 50, 255);
   rect(0, 0, width, height/2);
   sun();
   //if nothing has been loaded, print the instructions
   } else {
    fill(0);
    textFont(font, 30);
    text("Drag and Drop an mp3 to begin.", 30, 30, 800, 50);
   }
}

//ocean maker
void landscape() {
  //place the loaded image
  image(gradient, 0, height/2);
 /* this was an attempt to do the ocean without an image
  strokeWeight(5);
  for(int i = height; i> height/2 ; i -= 5) {
  stroke(i/4,i/4, 150 + i/4);
  line(0, i, width, i);
  }
  */
}

//sun animator
void sun() {
  if (r > -3) {
  pushMatrix();
  translate(width/2, height/2);
  rotate(r);
  fill(250 + r* 40, 250 + r *80, 20, 255 + 100 *(r+1)  );
  ellipse(400, 0, 100, 100);
  popMatrix();
  r -= .001;
  }
}

//Waves class for making waves
class Waves {
  float x;
 float y;
float radius;
float speed;
int A = 255;
 Waves() {
    
  }
  void update(float tx, float ty, float tradius) {
    x = tx;
    y = ty;
    radius = 20;
    speed = tradius/16;
    A = 255;
  }
  void display() {
   pushMatrix();
   translate(x,y);
   stroke(A, A, 255, A);
   strokeWeight(5);
   noFill();
   arc(0, 0, radius - y + 800 , radius , -2.64, -.5);
   popMatrix();
   y -= y/80;
   A -= 12;
   radius += speed;
  }
}

 
//here we load the file that has been dropped, and start the music and the analysis
void loadFile(File file) {
  song = Minim.loadFile(file.getPath());
  song.loop();
  fft = new FFT(song.mix.size(), 44100);
  fft.logAverages(22, 3);
  loaded = true;  
}
 // Everything froom here down was lifted from code in the Processing forums by jonbro
 
// DRAG'N'DROP
 
DropTarget dt = new DropTarget(this, new DropTargetListener() {
  public void dragEnter(DropTargetDragEvent event) {
    // debug messages for diagnostics
    //System.out.println("dragEnter " + event);
    event.acceptDrag(DnDConstants.ACTION_COPY);
  }
 
  public void dragExit(DropTargetEvent event) {
    //System.out.println("dragExit " + event);
  }
 
  public void dragOver(DropTargetDragEvent event) {
    //System.out.println("dragOver " + event);
    event.acceptDrag(DnDConstants.ACTION_COPY);
  }
 
  public void dropActionChanged(DropTargetDragEvent event) {
    //System.out.println("dropActionChanged " + event);
  }
 
  public void drop(DropTargetDropEvent event) {
    //System.out.println("drop " + event);
    event.acceptDrop(DnDConstants.ACTION_COPY);
 
    Transferable transferable = event.getTransferable();
    DataFlavor flavors[] = transferable.getTransferDataFlavors();
    int successful = 0;
 
    for (int i = 0; i < flavors.length; i++) {
 try {
   Object stuff = transferable.getTransferData(flavors[i]);
   if (!(stuff instanceof java.util.List)) continue;
   java.util.List list = (java.util.List) stuff;
 
   for (int j = 0; j < list.size(); j++) {
     Object item = list.get(j);
     if (item instanceof File) {
  File file = (File) item;
     
  // passing the whole file-object here
  loadFile(file);
     }
   }
 
 }  
 catch (Exception e) {
   e.printStackTrace();
 }
    }
  }
}
);

//this last part must be added before you try to play any songs, as it stops all the Minim processes

void stop()
{
  // always close Minim audio classes when you finish with them
  song.close();
  super.stop();
}
